/*
 Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
 Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
 Copyright (c) 1996-1998 The Open Group
 Copyright (c) 1994-1997 Sun Microsystems Inc.
 
 All rights reserved.
 This technology is a result of the Advanced Software  Enrichment 
 Project of Information-technology Promotion Agency, Japan (IPA).
 
 Permissions  to  use,  copy, modify and distribute this software
 are governed by the terms and conditions set forth in  the  file 
 COPYRIGHT, located in the top directory of this software.

 ************************************************************************

$Source: /togdev/cvsroot/adl2/src/examples/c/bank_example/bank.c,v $
$Revision: 1.2 $
$Date: 1998/02/27 15:22:47 $
 PRODUCT:	Assertion Definition Language Translation System
              (ADLTS) Version 2.0
 COMPONENT:	bank example (C)
 NAME:	bank.c
 	
 HISTORY:
 	Matt Evans, Sun Microsystems Inc. - Sep 27, 1996: Created
 		
 ************************************************************************/
#include "bank.h"


int         maxAccounts   = DEFAULT_MAX_ACCOUNTS;
bankAccount **accountList = NULL;
int         numAccts      = -1;

enum bankException bankError;
const int maxAccountsDefault = 16;



void
create_bank(int bankSize)
{

  int x;

  maxAccounts = (bankSize<=0) ? maxAccountsDefault :bankSize;
  numAccts    = 0;
  accountList = (bankAccount **) malloc (sizeof(bankAccount *) * maxAccounts);
  for (x = 0; x < maxAccounts;x++) {
    accountList[x] = NULL;
  }
}

bankAccount *
open_account(long amount, long *acctNumber)
{

  int x;

  if (amount < 0) {
    bankError = negativeAmountExc;
    return NULL;
  }
  if (numAccts < 0) {
    bankError = bankNotOpenExc;
    return NULL;
  }
  if (numAccts >= maxAccounts) {
    bankError = bankFullExc;
    return NULL;
  }
  for (x = 0;x<maxAccounts;x++) {

    /* find empty accountList index */

    if (accountList[x] == NULL) {
      *acctNumber  = x + 1;
      accountList[x] = createBankAccount(*acctNumber,amount);
      numAccts++;
      break;
    }
  }
  return (accountList[x]);
}

int
close_account(long acctNumber)
{

  int index = (int) acctNumber - 1;

  if (numAccts < 0) {
    bankError = bankNotOpenExc;
    return -1;
  }

  if (index < 0 || index >= maxAccounts) {
    bankError = invalidAccountNumberExc;
    return -1;
  }

  if (accountList[index] == NULL) {
    bankError = accountNotFoundExc;
    return -1;
  }

  free(accountList[index]);
  accountList[index] = NULL;
  return (--numAccts);
}
