/*
 Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
 Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
 Copyright (c) 1996-1998 The Open Group
 Copyright (c) 1994-1997 Sun Microsystems Inc.
 
 All rights reserved.
 This technology is a result of the Advanced Software  Enrichment 
 Project of Information-technology Promotion Agency, Japan (IPA).
 
 Permissions  to  use,  copy, modify and distribute this software
 are governed by the terms and conditions set forth in  the  file 
 COPYRIGHT, located in the top directory of this software.

 ************************************************************************

 SCCS:          @(#)bankAccount.h	1.1 11/19/96
 PRODUCT:	Assertion Definition Language Translation System
                (ADLTS) Version 2.0
 COMPONENT:	bank example ()
 NAME:   	bankAccount.h
 	
 HISTORY:
 	Matt Evans, Sun Microsystems Inc. - Sep 27, 1996: Created

 DESCRIPTION:

 declares the types and methods for the bankAccount interface.
 		
 ************************************************************************/
#ifndef BANKACCOUNT_H
#define BANKACCOUNT_H

#include "bankException.h"

#ifndef NULL
#define NULL (void *)0
#endif

#ifdef __ADL__
#include "bankAux.h"
#endif

typedef struct {

  long acct_number;
  long balance;

} bankAccount;


bankAccount *createBankAccount (long acct_num, long balance)
#ifdef __ADL__
  semantics
  [ abnormal = (return == NULL);
    normal   = (return != NULL);
  ]
  {
    @acct_num < 1 <:> bankError == invalidAccountNumberExc;
    @balance  < 0 <:> bankError == negativeAmountExc;

    if (normal) {
      return->balance     == balance;
      return->acct_number == acct_num;
    };
  }
  nld {
    acct_num            = "the specified account number";
    balance             = "the specified initial balance of the account";
    createBankAccount() = "create and initialize a bank account with its account number \
                           assign to $acct_num and its inital balance assigned to $balance",
                          &description ("the function creates and initalize a bank account"),
                          &includes("bank.h");
  }
#endif
;

long deposit (bankAccount *ba,long amt)
#ifdef __ADL__
  semantics
  [abnormal = return == -1;]
  {
    @ba == NULL <:> bankError == invalidAccountExc;
    @amt < 0    <:> bankError == negativeAmountExc;

    if (abnormal) {
      unchanged (get_balance(ba));
    };

    if (normal) {
      get_balance(ba) == @get_balance(ba) + amt;
      return == get_balance(ba);
    };
  }
  nld {
    ba        = "bank account";
    amt       = "deposited funds";
    deposit() = "deposit funds specified by $amt into the account $ba",
                &description ("the function deposits the funds into the account"),
                &includes("bank.h");
  }
#endif
;

long withdraw (bankAccount *ba,long amt)
#ifdef __ADL__
  semantics
  [abnormal = return == -1;]
  {
    @(amt < 0)               <:> bankError == negativeAmountExc;
    @(get_balance(ba) < amt) <:> bankError == insufficientFundsExc;

    if (abnormal) {
      unchanged (get_balance(ba));
    };

    if (normal) {
      get_balance(ba) == @get_balance(ba) - amt;
      return == get_balance();
    };
  }
  nld {
    ba         = "bank account";
    amt        = "withdrawn funds";
    withdraw() = "withdraw funds specified by $amt from the account $ba",
                 &description ("the function withdraw withdraws the funds from the account"),
                 &includes("bank.h");
  }
#endif
;
#endif
