// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
// Copyright (c) 1994-1997 Sun Microsystems Inc.
// 
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
// 
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
// 
// ************************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/examples/cpp/bank_example/bank.cpp,v $
// $Revision: 1.3 $
// $Date: 1998/02/27 13:05:35 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	bank example (C++)
// NAME:	bank.cpp
// 	
// HISTORY:
// 	Matt Evans, Sun Microsystems Inc. - Sep 27, 1996: Created
// 		
// ************************************************************************
#include "bank.hh"

bank::bank()
{
  bank(maxAccountsDefault); // default maximum number of accts
}

bank::bank(int bankSize) : numAccts(0)
{
  if (bankSize <= 0) {
    maxAccounts = maxAccountsDefault;
  }
  else {
    maxAccounts = bankSize;
  }
  accountList = (bankAccount **) new (bankAccount *[maxAccounts]);
  for (int x = 0; x < maxAccounts;x++) {
    accountList[x] = NULL;
  }
}

bank::~bank()
{
  for(int x=0;x<maxAccounts;x++) {
    if (accountList[x] != NULL) {
      delete accountList[x];
    }
  }
}


int
bank::get_max_accounts() {
return maxAccounts;
}

int
bank::get_active_accounts() {
return numAccts;
}

bankAccount *
bank::find_account(long acctNumber) throw
  (accountNotFoundExc)
{

  int index = (int) acctNumber - 1;

  if (index < 0 || index >= maxAccounts) {
    throw accountNotFoundExc();
  }
  if (accountList[index] == NULL) {
    throw accountNotFoundExc();
  }
  bankAccount *ba = accountList[index];
  return(ba);
}

bankAccount *
bank::open_account(long amount, long& acctNumber) throw
  (negativeAmountExc,
   bankFullExc)
{

  if (amount < 0) {
    throw negativeAmountExc();
  }
  if (numAccts < maxAccounts) {
    for (int x = 0;x<maxAccounts;x++) {
      if (accountList[x] == NULL) {
	numAccts++;
	acctNumber = x + 1;
	accountList[x] = new bankAccount(acctNumber,amount);
	//	acctNumber = x;
	return (accountList[x]);
      }
    }
  } else throw bankFullExc();
}

void
bank::close_account(long acctNumber) throw
  (accountNotFoundExc,
   accountNotZeroExc,
   invalidAccountExc)
{

  int index = (int) acctNumber - 1;

  if (index < 0 || index >= maxAccounts) {
    throw invalidAccountNumberExc();
  }
  if (accountList[index] == NULL) {
    throw accountNotFoundExc();
  }
  bankAccount *ba = accountList[index];
  close_account (ba);
}

void
bank::close_account (bankAccount *ba) throw
  (accountNotZeroExc,
   invalidAccountExc)
{

  if (ba == NULL) {
    throw invalidAccountExc();
  }

  int index = (int) ba->get_account_number() - 1;

  if (index < 0 || index >= maxAccounts || accountList[index] != ba) {
    throw invalidAccountExc();
  }

  if (ba->get_balance() != 0) {
    throw accountNotZeroExc();
  }

  delete ba;
  // remove it from the list

  accountList[index] = NULL;

  numAccts--;
}


void
bank::close_all_accounts() throw
  (invalidAccountExc,
   accountNotFoundExc,
   accountNotZeroExc)
{
  for (int x = 0; x < maxAccounts; x++) {
    if (accountList[x] != NULL) {
      close_account(accountList[x]);
    }
  }
}
