//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//  
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//  
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file
// COPYRIGHT, located in the top directory of this software.
//
//
// ************************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/examples/idl/bank_example/bankImpl.java,v $
// $Revision: 1.3 $
// $Date: 1998/02/27 15:46:32 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	bank example
// NAME:	bankImpl.java
// 	
// HISTORY:
// 	Cyril Inard d'Argence, The Open Group - Jan 13, 1998: Created
// 		
// ************************************************************************

package bank_example;

import bank_example.bank_app.*;

/**	
 *  class bankImpl
 *
 */

public class bankImpl extends bank_example.bank_app._bankImplBase {

  private int maxAccounts;	
  private bankAccount accountList[];
  private int numAccounts = 0;

  public bankImpl(){
    maxAccounts = maxAccountDefault;
    accountList = new bankAccount[maxAccounts];
  }

  public bankImpl(int bankSize) {
    maxAccounts = bankSize;
    accountList = new bankAccount[maxAccounts];
  }

  public int maxAccounts() {
    return maxAccounts;
  }

  public int numAccounts() {
    return numAccounts;
  }

  public int balance(int acctNumber)
        throws bank_example.bank_app.invalidAccountNumberExc, bank_example.bank_app.accountNotFoundExc
  {
     
     bankAccount ba = find_account(acctNumber);
     return ba.balance();
  }
  
  public bank_example.bank_app.bankAccount find_account(int acctNumber)
        throws bank_example.bank_app.invalidAccountNumberExc, bank_example.bank_app.accountNotFoundExc
  {

    int index = (int) acctNumber - 1;

    if (index < 0 || index >= maxAccounts) {
      throw (new invalidAccountNumberExc());
    }
    if (accountList[index] == null) {
      throw (new accountNotFoundExc());
    }
    return accountList[index];
  }

  public bank_example.bank_app.bankAccount open_account(int amount, 
                                          org.omg.CORBA.IntHolder acctNumber)
  throws bank_example.bank_app.negativeAmountExc, bank_example.bank_app.bankFullExc
  {
    if (amount < 0) {
      throw (new negativeAmountExc());
    }
    if (numAccounts >= maxAccounts) {
      throw (new bankFullExc());
    }

    bankAccount ret = null;
    for (int x=0;x<maxAccounts;x++) {
      if (accountList[x] == null) {
         accountList[x] = new bankAccountImpl(x + 1,amount);
         acctNumber.value = x + 1;
         numAccounts++;
         ret = accountList[x];
         break;
      }
    }
    return ret;
  }

  public void close_account(int acctNumber)
     throws bank_example.bank_app.invalidAccountNumberExc, 
            bank_example.bank_app.accountNotFoundExc,
            bank_example.bank_app.accountNotZeroExc
  {

    int index = (int) acctNumber - 1;

    if (index < 0 || index >= maxAccounts) {
      throw (new invalidAccountNumberExc());
    }
    if (accountList[index] == null) {
      throw (new accountNotFoundExc());
    }
    bankAccount ba = accountList[index];
    close_account (ba);
  }

  public void close_account (bankAccount ba)
    throws invalidAccountNumberExc,
           accountNotFoundExc,
           accountNotZeroExc {

    if (ba == null) {
      throw (new invalidAccountNumberExc());
    }

    int index = (int) ba.acct_number() - 1;

    if (index < 0 || index >= maxAccounts || accountList[index] == null) {
      throw (new invalidAccountNumberExc());
    }

    bankAccount my_acct =  accountList[index];

    if (   index != (int) my_acct.acct_number() - 1
        && ba.balance() != my_acct.balance()) {
      throw (new invalidAccountNumberExc());
    }

    // assume the input account is the same as the account
    // in the list.

    if (my_acct.balance() != 0) {
      throw (new accountNotZeroExc());
    }

    // remove it from the list

    accountList[index] = null;
    numAccounts--;
  }

  public void close_all_accounts()
    throws accountNotZeroExc
  {
    try
    {
       if (numAccounts > 0) {
          for (int x=0;x<maxAccounts;x++) {
             if (accountList[x] != null) {
                close_account(accountList[x]);
             }
          }
       }
    }
    catch (accountNotFoundExc e ) {} 
    catch (invalidAccountNumberExc e ) {}
  }
}

// ***********************************************************************
// History Log:
// $Log: bankImpl.java,v $
// Revision 1.3  1998/02/27 15:46:32  cinard
// Updated copyright notice.
//
// Revision 1.2  1998/02/12 16:04:16  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.1  1998/02/02 13:30:04  cinard
// Example of CORBA testing using ADL for Java.
//
// ***********************************************************************

