// Copyright(c) 1997-1998 The Open Group
// Copyright(c) 1994-1997 Sun Microsystems Inc.
// Copyright(c) 1994-1998 Information-technology Promotion Agency, Japan
//
// Permissions to use, copy, modify and distribute this software are
// governed by the terms and conditions set forth in the file COPYRIGHT,
// located in the ADLT release package.
//
//
// ***********************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/orbwrapper/vb/VbORBWrapper.java,v $ 
// $Revision: 1.4 $
// $Date: 1998/02/24 12:57:48 $ 
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   orbwrapper.sun
// NAME:        VbORBWrapper.java
//
// HISTORY:
//      Cyril Inard d'Argence, TOG - Jan 15,  1998: Created
//      See Log at end of file
//
// ***********************************************************************

import ORG.opengroup.adl.runtime.ORBWrapper;
import java.util.Properties;
import java.applet.Applet;
import org.omg.CORBA.ORB;
import org.omg.CORBA.portable.Skeleton;

/**
 * Concrete ORB wrapper for Visigenic's Visibroker Java ORB implementation.
 */
public class VbORBWrapper extends ORBWrapper
{

   /** The real ORB. */
   private ORB _orb = null; 

   /**
    * Create a new ORB instance for an application.  
    * @param args application strings from main() method.
    * @param props application-specific properties.
    * @return reference to the real ORB implementation.
    */
   final protected ORBWrapper _init(String args[], Properties props)
   { 
      _orb = ORB.init(args, props); 
      return this; 
   }

   /**
    * Create a new ORB instance for an applet.  
    * This method is not static so that it can be overrided in the real 
    * wrapper subclass.
    * @param app the applet.
    * @param props applet-specific properties.
    * @return reference to the real ORB implementation.
    */
   final protected ORBWrapper _init(Applet app, Properties props)
   { 
      _orb = ORB.init(app, props); 
      return this; 
   }

   /**
    * Convert a string to a CORBA object reference. 
    * @param str the string.
    * @return the object reference. 
    */
   final public Object string_to_object(String str)
   {
       return _orb.string_to_object(str);
   }

   /**
    * Convert a COBRA object reference to a string. 
    * @param the object reference to stringify. 
    * @return str the string.
    */
   final public String object_to_string(Object obj)
   {
       return _orb.object_to_string((org.omg.CORBA.Object)obj);
   }

   /**
    * All objects created by the user become ORB objects only after connect is
    * called. 
    * @param skeleton the object to connect.
    */
   final public void connect(Object skeleton) 
   {
     _orb.connect((Skeleton)skeleton);
   }

   /**
    * To stop an ORB object from receiving remote invocations disconnect needs
    * to be called.  
    * @param skeleton the object to disconnect.
    */
   final public void disconnect(Object skeleton) 
   {
     _orb.disconnect((Skeleton)skeleton);
   }
}

// ***********************************************************************
// History Log:
// $Log: VbORBWrapper.java,v $
// Revision 1.4  1998/02/24 12:57:48  cinard
// Updated import clause for the ORBWrapper class.
//
// Revision 1.3  1998/02/17 10:57:12  mallet
// Merged v1.0 with development trunk
//
// Revision 1.2.2.1  1998/02/16 20:26:08  cinard
// Concrete wrapper now in default java package.
//
// Revision 1.2  1998/02/12 16:01:59  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.1  1998/02/10 15:08:46  cinard
// Wrapper for Visigenic's Visibroker 3.1 Java ORB.
//
// ***********************************************************************
