//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inmode.c.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/runtime/c/ADLConcatenationSet.c,v $
// $Revision: 1.5 $
// $Date: 1998/07/21 13:48:56 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C Runtime Environment - Test directive and factory 
//                                      parameter set class
// NAME:	ADLConcatenationSet.c
// 	
// HISTORY:
//      Shane McCarron, The Open Group - 14 April 1998: Converted
//      Lionel Mallet, The Open Group RI  - Oct  26, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#include "ADLConcatenationSet.h"
#include "ADLMemory.h"
#include "ADLRuntimeException.h"

// the instance request should be starting at 0
static ADLDataSet *locateDataSet (ADLConcatenationSet *p, int instance) {
  int j;
  int num;
  ADLDataSet *d;
    
  // reset current cache
  p->mode.c.providedSet = -1;
  p->mode.c.providedIndex = -1;
  p->mode.c.providedSetPtr = NULL;
  
  // perform magic computation
  d = p->mode.c.elementSet;
  for (j = 0; j < p->mode.c.numElements; j++) {
    num = instance - d->numValues;
    
    if (num < 0 && instance >= 0) {
      p->mode.c.providedSet = j;
      p->mode.c.providedIndex = instance;
      return d;
    } else {
      instance = num;
      d = d->nextSet;
    }
  }
  
  // alas, we've got there. So we've not found an appropriate set
  return NULL;
}

static void checkBounds(ADLConcatenationSet *p, int instance) {
  // check to see if there are any elements
  if (p->mode.c.numElements == 0) {
    ADL_RaiseException(ADLRuntime,
			      "ADLConcatenationSet: numElements == 0");
  } else if (instance < 0 || instance >= p->numValues) {
    ADL_RaiseException(ADLRuntime,
			      "ADLConcatenationSet: index out of range (%d/%d)",
			      instance, p->numValues);
  }
}

static void *provide(ADLConcatenationSet *p, int index) {
  ADLDataSet *d;

  // check instance number
  checkBounds(p, index);

  if (d = locateDataSet(p, index)) {
    p->mode.c.providedSetPtr = d;
    return d->provide(d, p->mode.c.providedIndex);
  } else {
    // we should normally never execute this code
    return NULL;
  }
}

static int relinquish(ADLConcatenationSet *p, int index) {
  ADLDataSet *d;
  
  // check instance number
  checkBounds(p, index);

  if ((p->mode.c.providedSet != -1) && (p->mode.c.providedIndex != -1)) {
    d = p->mode.c.providedSetPtr;
    return d->relinquish(d, p->mode.c.providedIndex);
  } else {
    return 1;
  }
}

/*
 * Initialze the Concatenation Set - allocating memory to attach the data
 * sets to.
 */

ADLConcatenationSet *ADL_ConcatenationSetInit(char *label) {
  ADLConcatenationSet *set;

  set = ADL__calloc(1, sizeof(ADLConcatenationSet));
  if (set) {
      set->label = ADL__new_string(label);
      set->provide = &provide;
      set->relinquish = &relinquish;
      set->mode.c.elementSet = NULL;
  }
  return set;
}

void ADL_ConcatenationAddElement(ADLConcatenationSet *p, ADLDataSet *e) {
  ADLDataSet *ptr, *lptr;

  ptr = p->mode.c.elementSet;
  if (ptr == NULL) {
    p->mode.c.elementSet = e;
  } else {
    while (ptr) {
      lptr = ptr;
      ptr = ptr->nextSet;
    }
    
    lptr->nextSet = e;
  }

  // increase number of elements and values
  p->mode.c.numElements += 1;
  p->numValues += e->numValues;
}
// ***********************************************************************
// History Log:
// $Log: ADLConcatenationSet.c,v $
// Revision 1.5  1998/07/21 13:48:56  mallet
// Fixed typo in "concatenation".
// Replaced erroneous calls to ADL__getNumValues(set) with
// set->numValues: ADL__getNumValues is designed to be used with an
// ADLParameterSet argument.
// Changed label initialization to use ADL__new_string().
// Fixed bug in ADL_ConcatenationAddElement: incorrect behaviour for the
// first element added.
//
// Revision 1.4  1998/05/11 12:54:07  shane
// Changed _ADL_* symbols to ADL__* since our naming convention says that all ADL
// symbols should start with ADL or tdd.
//
// Revision 1.3  1998/05/08 20:21:05  shane
// Various modifications, including:
//
// Changed to print certain information only when debugMode is true.
// Changed to actually set debugMode based upon ADL_RPT_MODE in configuration.
// Fixed how pointers are used (man am I stupid).
//
// Revision 1.2  1998/04/29 15:58:38  shane
// Fixed my stupid unions that didn't work right.
//
// Revision 1.1  1998/04/16 12:09:09  shane
// First checkin of complete set.
//
//
// ***********************************************************************
