//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/runtime/cpp/ADLDataSet.hh,v $
// $Revision: 1.5 $
// $Date: 1998/02/27 10:01:03 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	C++ Runtime Environment - TDD datasets root class
// NAME:	ADLDataSet.hh
// 	
// HISTORY:
//      Lionel Mallet, The Open Group RI  - Oct  26, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#ifndef ADLDataSet_hh
#define ADLDataSet_hh

#include "ADLRuntimeException.hh"
#include <stdlib.h>
class ADLTest; // break circular dependency

/*
 *  abstract class ADLDataSet
 *
 *  The ADLDataSet abstract class defines the operations of
 *  all TDD data generation objects.
 *
 */
class ADLDataSet {

public:

  /*
   * Generic constructor
   *
   */
  ADLDataSet(void) : testContext(NULL), label(NULL) {}
  ADLDataSet(ADLTest *tc, char *label = "Anonymous") : testContext(tc), label(NULL), numValues(0) { setLabel(label); }

  /*
   * Returns the label of the element
   *
   * @return the string label
   */
  char *getLabel(void) { return this->label; }

  /*
   * Sets the label of the element
   */
  void setLabel(char *text);

  /*
   * Returns the potential number of values 
   * that this set element can yield.
   *
   * @return the number of values of this element
   */
  int getNumValues(void) { return numValues; }

  /*
   * Provides the object at the requested index. 
   *
   * @param index the index of the requested value (i.e., ordinal number)
   *
   * @return void * a pointer to the value at the index
   *
   * @exception ArrayIndexOutOfBoundsException The index is out of
   *            range of the number of potential instances of the
   *            set element.
   */
  virtual void *provide(int index) throw (ADLRuntimeException) = 0;

  /*
   * Relinquishes the object at the requested index. Returning
   * falses indicates an abort of the test. Not all
   * ADLDataSet objects will need to do anything in
   * relinquish.
   *
   * @param index the index of the requested value (i.e., ordinal number)
   *
   * @return boolean status of the relinquish (probably always true)
   *
   * @exception ArrayIndexOutOfBoundsException The index is out of
   *            range of the number of potential instances of the
   *            set element.
   */
  virtual bool relinquish(int index) throw (ADLRuntimeException) = 0;

protected:
  int numValues; // the number of values in the dataset

  virtual void checkBounds(int index) throw (ADLRuntimeException);

  ADLTest *testContext; // The test context where the dataset is 
  // used

private:
  char *label; // Dataset label
};

#endif // ADLDataSet_hh

// ***********************************************************************
// History Log:
// $Log: ADLDataSet.hh,v $
// Revision 1.5  1998/02/27 10:01:03  mallet
// Updated copyright notice.
//
// Revision 1.4  1998/02/19 09:45:47  mallet
// Changed signature and semantics of checkBounds to throw an
// ADLRuntimeException instead of returning a boolean.
// Changed signature of provide and relinquish to throw an
// ADLRuntimeException when checkBounds fails instead of returning NULL,
// which could be gently ignored.
//
// Revision 1.3  1998/02/12 16:04:26  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1997/11/21 16:45:38  mallet
// Replaced string by char* to become more portable.
// Replaced () by (void) in signature of method with no arguments.
// Added initialization of numValues in constructor.
//
// Revision 1.1  1997/11/14 10:13:29  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
//
// ***********************************************************************
