//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/runtime/cpp/ADLRuntimeException.cpp,v $
// $Revision: 1.5 $
// $Date: 1998/03/03 22:31:27 $
// PRODUCT:     Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:   C++ Runtime Environment - Runtime exception classes
// NAME:        ADLRuntimeException.cpp
//
// HISTORY:
//      Lionel Mallet, The Open Group RI - Jan 22, 1998: Created
//      See Log at end of file
//
// ***********************************************************************

#include "ADLRuntimeException.hh"
#include <string.h>
#include <stdio.h>


ADLRuntimeException::ADLRuntimeException() : type(ADLRuntime) {
  init_detail_string(type);
}

ADLRuntimeException::ADLRuntimeException(ADLRuntimeExceptionType t, 
					 char *d) {
  type = t;
  init_detail_string(type);

  if (d) {
    if (detail) {
      char *tmp = new char[strlen(detail) + strlen(d) + 3];
      sprintf(tmp, "%s: %s", detail, d);
      delete detail;
      detail = tmp;
    } else {
      detail = new char [strlen(d) + 1];
      strcpy(detail, d);
    }
  }
}

ADLRuntimeException::ADLRuntimeException(ADLRuntimeExceptionType t) {
    ADLRuntimeException(t, (char *)NULL);
}


ADLRuntimeException::ADLRuntimeException(const ADLRuntimeException& e) {
  type = e.type;
  if (e.detail) {
    detail = new char[strlen(e.detail) + 1];
    strcpy(detail, e.detail);
  }
}

ADLRuntimeException::~ADLRuntimeException() {
  if (detail) {
    delete detail;
  }
}

void ADLRuntimeException::init_detail_string(ADLRuntimeExceptionType t) {
  static char *msg[2] = { "Ending test case", "Ending test" };

  switch (t) {
  case ADLRuntime:
    detail = 0;
    break;
  case ADLEndCase:
    detail = new char[strlen(msg[0]) + 1];
    strcpy(detail, msg[0]);
    break;
  case ADLEndTest:
    detail = new char[strlen(msg[1]) + 1];
    strcpy(detail, msg[1]);
    break;
  }
}

// ***********************************************************************
// History Log:
// $Log: ADLRuntimeException.cpp,v $
// Revision 1.5  1998/03/03 22:31:27  shane
// Added handling for WIN32-isms.
// Now compiles cleanly on Win32 - don't know if it runs.
//
// Revision 1.4  1998/02/27 10:01:05  mallet
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:04:29  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1998/01/23 14:50:07  mallet
// Fixed copy constructor signature: should be const ADLRuntimeException&.
//
// Revision 1.1  1998/01/23 10:05:12  mallet
// Exceptions raised from the runtime to notify an EndCase or EndTest
// request from the user. Currently EndTest is partially supported since
// there is no way to notify TET to abort the test execution.
// Due to some limitations in the g++, there is only a single exception
// class with a discriminative field indicating its subtype.
//
// ***********************************************************************
