//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
// ***********************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/runtime/cpp/ADLString.hh,v $
// $Revision: 1.6 $
// $Date: 1998/05/29 12:24:29 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	
// NAME:	ADLString.hh
// 	
// HISTORY:
// 	Lionel Mallet, The Open Group RI - Oct 28, 1997: Created
//      See Log at end of file
// 		
// ***********************************************************************

#ifndef ADLString_hh
#define ADLString_hh

#ifndef WIN32
#include <strstream.h>
#else
#include <strstrea.h>
#endif
#include <string.h>

/*
 * class ADLString
 *
 * The ADLString class extends the ostrstream class. It adds a couple of 
 * user-defined conversions from/to char * and overloads the + operator
 * to add Java-like building of a string from different values
 * This class should never be instantiated: NO VARIABLE DECLARATION!
 * It should be used as istream and ostream through a static instance
 * named `adlstr'.
 * Typical usage: printf(adlstr.reset() << "something!");
 */
class ADLString : public ostrstream {
public:
  ADLString(void) : nullTerminated(false) {} // empty default constructor
  // the constructor below is normally never used.
  //  ADLString(char *init) { this->reset() << init; }

  ~ADLString(void); // default destructor

  operator char*(void); // conversion to char *

  // redefine str to be sure that the string is null terminated
  char *str(void);

  // method to reinitialize the string buffer
  ADLString& reset(void);

  // redefined operator
  ADLString& operator<<(char *s);
  ADLString& operator<<(const char *s);
  ADLString& operator<<(const unsigned char *s)
    { return operator<<((const char*)s); }
  ADLString& operator<<(const signed char *s)
    { return operator<<((const char*)s); }
  ADLString& operator<<(char c);
  ADLString& operator<<(unsigned char c) { return operator<<((char)c); }
  ADLString& operator<<(signed char c) { return operator<<((char)c); }
  ADLString& operator<<(int i);
  ADLString& operator<<(unsigned int i);
  ADLString& operator<<(short i) { return operator<<((int)i); }
  ADLString& operator<<(unsigned short i)
    { return operator<<((unsigned int)i); }
  ADLString& operator<<(long l);
  ADLString& operator<<(unsigned long l);
#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
  ADLString& operator<<(long long l);
  ADLString& operator<<(unsigned long long l);
#endif
  ADLString& operator<<(const void *v);
#ifdef _G_HAVE_BOOL
  ADLString& operator<<(bool b) { return operator<<((int)b); }
#endif
  ADLString& operator<<(double d);
  ADLString& operator<<(float f) { return operator<<((double)f); }
#ifdef __linux__
  ADLString& operator<<(long double d);
#else
  ADLString& operator<<(long double d) { return operator<<((double)d); }
#endif
  ADLString& operator<<(ostream& (*func)(ostream&));
  ADLString& operator<<(ios& (*func)(ios&));
  ADLString& operator<<(streambuf* b);

private:
  bool nullTerminated;
};

extern ADLString adlstr;

#endif // ADLString_hh

// ***********************************************************************
// History Log:
// $Log: ADLString.hh,v $
// Revision 1.6  1998/05/29 12:24:29  shane
// dded an include of string.h.
//
// Revision 1.5  1998/03/03 22:31:28  shane
// Added handling for WIN32-isms.
// Now compiles cleanly on Win32 - don't know if it runs.
//
// Revision 1.4  1998/02/27 10:01:06  mallet
// Updated copyright notice.
//
// Revision 1.3  1998/02/12 16:04:30  mallet
// Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
// and IPA.
//
// Revision 1.2  1997/11/21 17:03:49  mallet
// Fixed prototypes for method with no arguments: void required.
//
// Revision 1.1  1997/11/14 10:13:31  mallet
// Initial version of runtime classes for TDD datasets and ACOs.
//
//
// ***********************************************************************
