//
// Copyright (c) 1996-1998 Information-technology Promotion Agency, Japan
// Copyright (c) 1996-1998 Mitsubishi Research Institute, Inc.
// Copyright (c) 1996-1998 The Open Group
//
// All rights reserved.
// This technology is a result of the Advanced Software  Enrichment 
// Project of Information-technology Promotion Agency, Japan (IPA).
//
// Permissions  to  use,  copy, modify and distribute this software
// are governed by the terms and conditions set forth in  the  file 
// COPYRIGHT, located in the top directory of this software.
//
//************************************************************************
//
// $Source: /togdev/cvsroot/adl2/src/runtime/cpp/ADLTestCase.cpp,v $
// $Revision: 1.6 $
// $Date: 1998/02/27 10:01:07 $
// PRODUCT:	Assertion Definition Language Translation System
//              (ADLTS) Version 2.0
// COMPONENT:	
// NAME:	ADLTestCase.cpp
// 	
// HISTORY:
// 	Shane P. McCarron, The Open Group - OCt 21, 1997: Created
//      See Log at end of file
//
// NOTES:
// 	Note that this implementation assumes that there is an instance of the
// 	ADLTest class or one of its children that is linked with the
// 	implementation. This class needs to have a runTestInstance method that
// 	can do the provide function calls and then exercise the actual
// 	ACO.
//
// 	Also note that this implementation can be used in the C binding as
// 	well - that is why the cplusplus declarations are conditional, and why
// 	the macros are defined as they are for doing the mapping to the
// 	generated C test case.
// 		
//****************z*******************************************************

#ifdef __cplusplus
extern "C" {
#endif

#include "ADLTestCase.h"
#ifndef TET_PASS
#include "tet_api.h"
#endif

/**
 * Return the minimum IC number for the test
 */

int tet_getminic() {
    return 1;
}

/**
 * Return the maximum IC number for the test
 */

int tet_getmaxic() { 
    return ADL_GETNUMINSTANCES();
}

/**
 * Return the number of this test
 */

int tet_gettestnum(int ic, int tp) {
    return(ic);
}

/**
 * Return the total number of test purposes.
 */

int tet_gettpcount(int ic) {
    return(1);
}

/**
 * Return whether the referenced IC is valid.
 *
 * @param ic	The IC to check.
 */

int tet_isdefic(int ic) {
if ((ic < tet_getminic()) || (ic > tet_getmaxic()))
	return 0;
else
	return 1;
}

/**
 * Execute a test purpose by calling the runTestInstance method of the ADLTest
 * class.
 */

int tet_invoketp(int ic, int tpid)
{
  // the ic number needs to be reduced by one here since instance
  // number starts from 0 in ADL but TET doesn't like IC number 
  // starting from 0.
  ADL_RUNTEST(ic - 1);

  return 0;
}

void (*tet_cleanup)() = NULL;
void (*tet_startup)() = NULL;

#ifdef __GNUG__
struct tet_testlist tet_testlist[] = {};

void terminate () {

  tet_infoline ("terminate called, program aborting");
  tet_logoff();
  abort();

}

void unexpected() {

  tet_infoline ("Uncaught Exception occured");

}

#else
struct tet_testlist tet_testlist[] = { NULL };
#endif

#ifdef __cplusplus
}
#endif

/***********************************************************************
 * History Log:
 * $Log: ADLTestCase.cpp,v $
 * Revision 1.6  1998/02/27 10:01:07  mallet
 * Updated copyright notice.
 *
 * Revision 1.5  1998/02/12 16:04:31  mallet
 * Fixed copyright notice: TOG RI -> TOG, year extended to 1998 for TOG
 * and IPA.
 *
 * Revision 1.4  1998/01/19 14:37:41  mallet
 * Added #ifndef TET_PASS around tet_api.h include to avoid duplicate
 * definition of tet types.
 * Fixed initialization of tet_testlist in ADLTestCase.cpp for GCC.
 *
 * Revision 1.3  1997/11/21 17:07:00  mallet
 * Added a return statement for method tet_invoketp which is declared to
 * return an int.
 * Fixed initialization of tet_testlist for Sun's C++ compiler: requires
 * { NULL } instead of just {}.
 *
 * Revision 1.2  1997/11/14 10:25:46  mallet
 * Added missing tet_cleanup, tet_startup and tet_testlist
 * initializations.
 * Added fallback functions for exception handling: terminate and unexpected.
 *
 * Revision 1.1  1997/10/22 11:27:59  shane
 * Initial version.
 *
 *
 */
